Option Strict On
Option Explicit On 

Imports System
Imports System.Diagnostics


Public Enum TraceIssues
   ParamCanNotBeNull
   ParamLessThan
   ParamGreaterThan
   ParamInRange_High
   ParamInRange_Low
   ParamStringRequired
   StartingMethod
   InCondition
   InFor
   MarkSpot
End Enum

Public Class Tools
#Region "Class Level Fields"
   Private Const crlf As String = Microsoft.VisualBasic.ControlChars.CrLf
   Private Const tab As String = Microsoft.VisualBasic.ControlChars.Tab

   Private Shared mbKnownListeners As Boolean
   Private Shared mRunGUID As System.Guid

   Private Shared TSLevel As TraceSwitch = _
       New TraceSwitch("TraceLevelSwitch", "Trace Level Test Switch")
   Private Shared TSSkipEventArgs As BooleanSwitch = _
       New BooleanSwitch("SkipEventArgs", "Skip Event Arg details")
   Private Shared TSSkipArguments As BooleanSwitch = _
       New BooleanSwitch("SkipArguments", "Skip Argument details")
#End Region

#Region "Management Functions"
   Shared Sub New()
      mRunGUID = System.Guid.NewGuid
   End Sub

   Public Shared Property SkipEventArgs() As Boolean
      Get
         Return TSSkipEventArgs.Enabled
      End Get
      Set(ByVal Value As Boolean)
         TSSkipEventArgs.Enabled = Value
      End Set
   End Property

   Public Shared Property SkipArguments() As Boolean
      Get
         Return TSSkipArguments.Enabled
      End Get
      Set(ByVal Value As Boolean)
         TSSkipArguments.Enabled = Value
      End Set
   End Property

   Public Shared Property TraceLevel() As TraceLevel
      Get
         Return TSLevel.Level
      End Get
      Set(ByVal Value As TraceLevel)
         TSLevel.Level = Value
      End Set
   End Property

   Public Shared Sub ClearListeners()
      Trace.Listeners.Clear()
   End Sub

   Public Shared Sub AddListener(ByVal traceListener As TraceListener)
      mbKnownListeners = False
      Trace.Listeners.Add(traceListener)
   End Sub

   Public Shared Sub RemoveListener(ByVal tracelistener As TraceListener)
      ' ENHANCE: If you're removing listeners created at the command prompt
      '          you'll need to add a name based remove process
      mbKnownListeners = False
      Trace.Listeners.Remove(tracelistener)
   End Sub

   Private Shared Function HasListeners() As Boolean
      Return Trace.Listeners.Count > 0
   End Function
#End Region


#Region "Sane parameter checking supporting Trace Levels (Trace Switches)"
   'NOTE: You can not refactor these public methods to call each other, or you will
   '      lose track of the stackframe

#Region "Param Can't be Nothing"
   Public Shared Sub ParamCanNotBeNull( _
               ByVal paramName As String, _
               ByVal obj As Object)
      Dim traceLevel As traceLevel = traceLevel.Error
      If (TSLevel.Level >= traceLevel) AndAlso Trace.Listeners.Count > 0 Then
         If (obj Is Nothing) Then
            ParamCanNotBeNullFailed(paramName, _
             traceLevel.Error, obj, New StackFrame(1, True))
         End If
      End If
   End Sub
   Public Shared Sub ParamCanNotBeNull( _
               ByVal paramName As String, _
               ByVal traceLevel As TraceLevel, _
               ByVal obj As Object)
      If (TSLevel.Level >= traceLevel) AndAlso Trace.Listeners.Count > 0 Then
         If (obj Is Nothing) Then
            ParamCanNotBeNullFailed(paramName, _
             traceLevel, obj, New StackFrame(1, True))
         End If
      End If
   End Sub
   Private Shared Sub ParamCanNotBeNullFailed( _
               ByVal paramName As String, _
               ByVal traceLevel As TraceLevel, _
               ByVal obj As Object, _
               ByVal stackframe As StackFrame)
      TraceEntry(stackframe, traceLevel, Nothing, paramName, _
                  Nothing, TraceIssues.ParamCanNotBeNull, _
                  "Parameter " + paramName + " can not be Nothing")
   End Sub
#End Region

#Region "Param Less Than"
   Public Shared Sub ParamLessThan( _
               ByVal paramName As String, _
               ByVal val As Int32, _
               ByVal HighBound As Int32)
      Dim traceLevel As traceLevel = traceLevel.Error
      ' TODO: sanity check for HighBound < highBound
      If (TSLevel.Level >= traceLevel) AndAlso Trace.Listeners.Count > 0 Then
         If (val > HighBound) Then
            ParamLessThanFailed(paramName, _
             traceLevel, val, HighBound, _
             New StackFrame(1, True))
         End If
      End If
   End Sub
   Public Shared Sub ParamLessThan( _
               ByVal paramName As String, _
               ByVal traceLevel As TraceLevel, _
               ByVal val As Int32, _
               ByVal HighBound As Int32)
      ' TODO: sanity check for HighBound < highBound
      If (TSLevel.Level >= traceLevel) AndAlso Trace.Listeners.Count > 0 Then
         If (val > HighBound) Then
            ParamLessThanFailed(paramName, _
             traceLevel, val, HighBound, _
             New StackFrame(1, True))
         End If
      End If
   End Sub
   Private Shared Sub ParamLessThanFailed( _
               ByVal paramName As String, _
               ByVal traceLevel As TraceLevel, _
               ByVal val As Int32, _
               ByVal HighBound As Int32, _
               ByVal stackframe As StackFrame)
      TraceEntry(stackframe, traceLevel, val, paramName, _
                Nothing, TraceIssues.ParamLessThan, _
               "Parameter " + paramName + " must be less than " + _
               HighBound.ToString())
   End Sub
#End Region

#Region "Param Greater Than"
   Public Shared Sub ParamGreaterThan( _
               ByVal paramName As String, _
               ByVal val As Int32, _
               ByVal LowBound As Int32)
      Dim traceLevel As traceLevel = traceLevel.Error
      ' TODO: sanity check for LowBound < LowBound
      If (TSLevel.Level >= traceLevel) AndAlso Trace.Listeners.Count > 0 Then
         If (val < LowBound) Then
            ParamGreaterThanFailed(paramName, _
             traceLevel, val, LowBound, _
             New StackFrame(1, True))
         End If
      End If
   End Sub
   Public Shared Sub ParamGreaterThan( _
               ByVal paramName As String, _
               ByVal traceLevel As TraceLevel, _
               ByVal val As Int32, _
               ByVal LowBound As Int32)
      ' TODO: sanity check for LowBound < LowBound
      If (TSLevel.Level >= traceLevel) AndAlso Trace.Listeners.Count > 0 Then
         If (val < LowBound) Then
            ParamGreaterThanFailed(paramName, _
             traceLevel, val, LowBound, _
             New StackFrame(1, True))
         End If
      End If
   End Sub
   Private Shared Sub ParamGreaterThanFailed( _
               ByVal paramName As String, _
               ByVal traceLevel As TraceLevel, _
               ByVal val As Int32, _
               ByVal LowBound As Int32, _
               ByVal stackframe As StackFrame)
      TraceEntry(stackframe, traceLevel, val, paramName, _
                Nothing, TraceIssues.ParamGreaterThan, _
               "Parameter " + paramName + " must be greater than " + _
               LowBound.ToString())
   End Sub
#End Region

#Region "Param In Range"
   Public Shared Sub ParamInRange( _
               ByVal paramName As String, _
               ByVal val As Int32, _
               ByVal lowBound As Int32, _
               ByVal highBound As Int32)
      Dim traceLevel As traceLevel = traceLevel.Error
      ' TODO: sanity check for lowBound < highBound
      If (TSLevel.Level >= traceLevel) AndAlso Trace.Listeners.Count > 0 Then
         If (val < lowBound) Then
            ParamInRangeFailedLow(paramName, _
             traceLevel, val, lowBound, highBound, _
             New StackFrame(1, True))
         ElseIf (val > highBound) Then
            ParamInRangeFailedHigh(paramName, _
             traceLevel, val, lowBound, highBound, _
             New StackFrame(1, True))
         End If
      End If
   End Sub
   Public Shared Sub ParamInRange( _
               ByVal paramName As String, _
               ByVal traceLevel As TraceLevel, _
               ByVal val As Int32, _
               ByVal lowBound As Int32, _
               ByVal highBound As Int32)
      ' TODO: sanity check for lowBound < highBound
      If (TSLevel.Level >= traceLevel) AndAlso Trace.Listeners.Count > 0 Then
         If (val < lowBound) Then
            ParamInRangeFailedLow(paramName, _
             traceLevel, val, lowBound, highBound, _
             New StackFrame(1, True))
         ElseIf (val > highBound) Then
            ParamInRangeFailedHigh(paramName, _
             traceLevel, val, lowBound, highBound, _
             New StackFrame(1, True))
         End If
      End If
   End Sub
   Private Shared Sub ParamInRangeFailedLow( _
               ByVal paramName As String, _
               ByVal traceLevel As TraceLevel, _
               ByVal val As Int32, _
               ByVal lowBound As Int32, _
               ByVal highBound As Int32, _
               ByVal stackframe As StackFrame)
      TraceEntry(stackframe, traceLevel, val, paramName, _
                Nothing, TraceIssues.ParamInRange_Low, _
               "Parameter " + paramName + " must be greater than " + _
               lowBound.ToString())
   End Sub
   Private Shared Sub ParamInRangeFailedHigh( _
               ByVal paramName As String, _
               ByVal traceLevel As TraceLevel, _
               ByVal val As Int32, _
               ByVal lowBound As Int32, _
               ByVal highBound As Int32, _
               ByVal stackframe As StackFrame)
      TraceEntry(stackframe, traceLevel, val, paramName, _
               Nothing, TraceIssues.ParamInRange_Low, _
               "Parameter " + paramName + " must be less than " + _
               highBound.ToString())
   End Sub
#End Region

#Region "Param Required"
   Public Shared Sub ParamStringRequired( _
               ByVal paramName As String, _
               ByVal val As String)
      Dim traceLevel As traceLevel = traceLevel.Error
      If (TSLevel.Level >= traceLevel) AndAlso Trace.Listeners.Count > 0 Then
         If (val Is Nothing OrElse val = "") Then
            ParamStringRequiredFailed(paramName, traceLevel, val, New StackFrame(1, True))
         End If
      End If
   End Sub
   Public Shared Sub ParamStringRequired( _
               ByVal paramName As String, _
               ByVal traceLevel As TraceLevel, _
               ByVal val As String)
      If (TSLevel.Level >= traceLevel) AndAlso Trace.Listeners.Count > 0 Then
         If (val Is Nothing OrElse val Is "") Then
            ParamStringRequiredFailed(paramName, traceLevel, val, New StackFrame(1, True))
         End If
      End If
   End Sub
   Private Shared Sub ParamStringRequiredFailed( _
               ByVal paramName As String, _
               ByVal traceLevel As TraceLevel, _
               ByVal val As String, _
               ByVal stackframe As StackFrame)
      TraceEntry(stackframe, traceLevel, val, paramName, _
                Nothing, TraceIssues.ParamStringRequired, _
               "Parameter " + paramName + " must have a length > 0 ")
   End Sub
#End Region

#Region "Starting Event"
   Public Shared Sub StartingEvent( _
               ByVal sender As System.Object, _
               ByVal e As System.EventArgs)
      Dim traceLevel As traceLevel = traceLevel.Info
      If (TSLevel.Level >= traceLevel) AndAlso Trace.Listeners.Count > 0 Then
         StartingEvent(sender, e, traceLevel, New StackFrame(1, True))
      End If
   End Sub
   Public Shared Sub StartingEvent( _
               ByVal sender As System.Object, _
               ByVal e As System.EventArgs, _
               ByVal traceLevel As TraceLevel)
      If (TSLevel.Level >= traceLevel) AndAlso Trace.Listeners.Count > 0 Then
         StartingEvent(sender, e, traceLevel, New StackFrame(1, True))
      End If
   End Sub
   Private Shared Sub StartingEvent( _
               ByVal sender As System.Object, _
               ByVal e As System.EventArgs, _
               ByVal traceLevel As TraceLevel, _
               ByVal stackframe As StackFrame)
      Dim name As String
      Dim properties As String = ""
      If TypeOf sender Is Windows.forms.Control Then
         name = CType(sender, Windows.Forms.Control).Name
      Else
         name = sender.ToString
      End If
      If Not TSSkipEventArgs.Enabled Then
         ' Only do the reflection if you know its wanted
         For Each p As Reflection.PropertyInfo In e.GetType.GetProperties
            properties &= p.Name & ":" & p.GetValue(e, Nothing).ToString & ", " & crlf
         Next
      End If
      TraceEntry(stackframe, traceLevel, Nothing, "", _
                  Nothing, TraceIssues.StartingMethod, _
                  "Starting Event - Called By: " & name & crlf & tab & _
                           "Properties: " & properties)
   End Sub
#End Region

#Region "Starting Method"
   ' This design will cause problems if the first value passed to the method 
   ' is a TraceLevel. I'm ignoring that side case
   Public Shared Sub StartingMethod(ByVal ParamArray params() As Object)
      Dim traceLevel As traceLevel = traceLevel.Info
      If (TSLevel.Level >= traceLevel) AndAlso Trace.Listeners.Count > 0 Then
         StartingMethod(params, traceLevel, New StackFrame(1, True))
      End If
   End Sub
   Public Shared Sub StartingMethod( _
               ByVal traceLevel As TraceLevel, _
               ByVal ParamArray params() As Object)
      If (TSLevel.Level >= traceLevel) AndAlso Trace.Listeners.Count > 0 Then
         StartingMethod(params, traceLevel, New StackFrame(1, True))
      End If
   End Sub
   Private Shared Sub StartingMethod( _
               ByVal params() As Object, _
               ByVal traceLevel As TraceLevel, _
               ByVal stackframe As StackFrame)
      TraceEntry(stackframe, traceLevel, Nothing, "", _
               params, TraceIssues.StartingMethod, _
               "Starting Method ")
   End Sub
#End Region

#Region "In Condition"
   Public Shared Sub InCondition( _
               ByVal description As String)
      Dim traceLevel As traceLevel = traceLevel.Verbose
      If (TSLevel.Level >= traceLevel) AndAlso Trace.Listeners.Count > 0 Then
         InCondition(description, traceLevel, New StackFrame(1, True))
      End If
   End Sub
   Public Shared Sub InCondition( _
               ByVal description As String, _
               ByVal traceLevel As TraceLevel)
      If (TSLevel.Level >= traceLevel) AndAlso Trace.Listeners.Count > 0 Then
         InCondition(description, traceLevel, New StackFrame(1, True))
      End If
   End Sub
   Private Shared Sub InCondition( _
               ByVal description As String, _
               ByVal traceLevel As TraceLevel, _
               ByVal stackframe As StackFrame)
      TraceEntry(stackframe, traceLevel, Nothing, "", _
                  Nothing, TraceIssues.InCondition, _
                  "In Condition: " + description)
   End Sub
#End Region

#Region "In For"
   Public Shared Sub InFor( _
               ByVal val As Int32)
      Dim traceLevel As traceLevel = traceLevel.Verbose
      If (TSLevel.Level >= traceLevel) AndAlso Trace.Listeners.Count > 0 Then
         InFor("In For", val, traceLevel, New StackFrame(1, True))
      End If
   End Sub
   Public Shared Sub InFor( _
               ByVal val As Int32, _
               ByVal traceLevel As TraceLevel)
      If (TSLevel.Level >= traceLevel) AndAlso Trace.Listeners.Count > 0 Then
         InFor("In For", val, traceLevel, New StackFrame(1, True))
      End If
   End Sub
   Public Shared Sub InFor( _
               ByVal description As String, _
               ByVal val As Int32)
      Dim traceLevel As traceLevel = traceLevel.Verbose
      If (TSLevel.Level >= traceLevel) AndAlso Trace.Listeners.Count > 0 Then
         InFor(description, val, traceLevel, New StackFrame(1, True))
      End If
   End Sub
   Public Shared Sub InFor( _
               ByVal description As String, _
               ByVal val As Int32, _
               ByVal traceLevel As TraceLevel)
      If (TSLevel.Level >= traceLevel) AndAlso Trace.Listeners.Count > 0 Then
         InFor(description, val, traceLevel, New StackFrame(1, True))
      End If
   End Sub
   Private Shared Sub InFor( _
               ByVal description As String, _
               ByVal val As Int32, _
               ByVal traceLevel As TraceLevel, _
               ByVal stackframe As StackFrame)
      TraceEntry(stackframe, traceLevel, val, "", _
                Nothing, TraceIssues.InFor, _
               "In For: " + description + "(" + val.ToString + ")")
   End Sub
#End Region

#Region "Mark the Spot"
   Public Shared Sub MarkSpot( _
               ByVal description As String)
      Dim traceLevel As traceLevel = traceLevel.Verbose
      If (TSLevel.Level >= traceLevel) AndAlso Trace.Listeners.Count > 0 Then
         MarkSpot(description, traceLevel, New StackFrame(1, True))
      End If
   End Sub
   Public Shared Sub MarkSpot( _
               ByVal description As String, _
               ByVal traceLevel As TraceLevel)
      If (TSLevel.Level >= traceLevel) AndAlso Trace.Listeners.Count > 0 Then
         MarkSpot(description, traceLevel, New StackFrame(1, True))
      End If
   End Sub
   Private Shared Sub MarkSpot( _
               ByVal description As String, _
               ByVal traceLevel As TraceLevel, _
               ByVal stackframe As StackFrame)
      TraceEntry(stackframe, traceLevel, Nothing, "", _
                Nothing, TraceIssues.MarkSpot, _
               "Mark Spot:" + description)
   End Sub
#End Region

#End Region

#Region "Define Output "
   Private Shared Sub TraceEntry( _
               ByVal stackFrame As StackFrame, _
               ByVal traceLevel As TraceLevel, _
               ByVal value As Object, _
               ByVal valueName As String, _
               ByVal params() As Object, _
               ByVal traceIssue As TraceIssues, _
               ByVal issue As String)
      Dim traceEntry As traceEntry
      traceEntry = New traceEntry(mRunGUID, stackFrame, params, _
                     traceLevel, value, valueName, traceIssue, issue)
      Trace.WriteLine(traceEntry)
   End Sub
#End Region

#Region "Private support routines"
   Private Shared Function ShouldTrace(ByVal traceLevel As TraceLevel) As Boolean
      If (TSLevel.Level >= traceLevel) Then
         Return HasListeners()
      End If
   End Function
#End Region
End Class
